# -*- coding: utf-8 -*-

import grok

from zope.component import getMultiAdapter, queryAdapter
from zope.interface import Interface
from zope.location.interfaces import ISite
from waeup.sirp.browser.pages import (
    UniversityPage, FacultyContainerPage, DatacenterPage, FacultyPage,
    DepartmentPage, CoursePage, CertificatePage, CertificateCoursePage, UserContainerPage)
from waeup.sirp.browser.interfaces import (
    IFacultyContainer, IFaculty, IDepartment, ICourse, ICertificate,
    ICertificateCourse, IBreadcrumbContainer, IUniversity, IUserContainer)
from waeup.sirp.interfaces import (IWAeUPObject, IWAeUPXMLExporter,
                                   IWAeUPXMLImporter, IDataCenter)
from waeup.sirp.browser.layout import WAeUPPage

grok.templatedir('templates')
grok.context(IWAeUPObject) # Make IWAeUPObject the default context

class LeftSidebar(grok.ViewletManager):
    grok.name('left')

class BreadCrumbManager(grok.ViewletManager):
    grok.name('breadcrumbs')

class ActionBar(grok.ViewletManager):
    grok.name('actionbar')

class AdministrationTasks(grok.ViewletManager):
    grok.name('admintasks')


#
# Baseclasses that give some defaults for really used viewlets.
#
class ActionButton(grok.Viewlet):
    """A base for action buttons.

    An action button provides an icon, some text and links to a
    target.  If you want to set a different text, icon or target name
    for some active button below, just override the approriate
    attribute in the concerned viewlet.

    Action buttons provide by default dynamic attributes

     * ``alt``
          An alternative text for the icon. By default the same as
          the text.

     * ``icon_url``
          The URL of the icon.

     * ``target_url``
          The URL of the link target.

    """
    grok.baseclass()
    grok.context(IWAeUPObject)
    grok.viewletmanager(ActionBar)
    icon = 'actionicon_modify.png' # File must exist in static/
    target = '@@manage' # link to this viewname.
    text = 'Edit' # Text to display on the button

    # We set the template file explicitly (instead of using
    # ``grok.template('actionbutton')``) to stick with this template
    # also in derived classes in other packages. If we didn't, those
    # derived ActionButton viewlets had to provide an own template,
    # which would not be updated automatically, when the local
    # template ``templates/actionbutton.pt`` changes.
    #
    # Inheriting viewlets that wish to use their own template anyway
    # can do so by setting their local ``grok.template(<mytemplate>)``
    # and setting ``template`` to ``None`` for the class::
    #
    # class DerivedActionButton(ActionButton):
    #   ...
    #   grok.template('overriding_template')
    #   template = None
    #   ...
    #
    template = grok.PageTemplateFile('templates/actionbutton.pt')

    @property
    def alt(self):
        """Alternative text for icon.
        """
        return self.text

    @property
    def icon_url(self):
        """Get the icon URL.
        """
        static = self.view.static
        if static is None or static.get(self.icon, None) is None:
            # In derived classes defined in other modules/packages
            # than w.s.browser, ``static`` might refer to a static dir
            # local to the derived class' module. As we often like to
            # get the icons from here
            # (i.e. waeup.sirp.browser/static), we set the directory
            # resource appropiately.
            #
            # XXX: The hardcoding of 'w.s.browser' should be replaced
            #      by something smarter.
            #
            # TODO: notes in here should go to general documentation.
            static = queryAdapter(
                self.request, Interface, name='waeup.sirp.browser')
        return static[self.icon]()

    @property
    def target_url(self):
        """Get a URL to the target...
        """
        return self.view.url(self.view.context, self.target)

class PlainActionButton(ActionButton):
    """A base for action buttons without image
    """
    grok.baseclass()
    template = grok.PageTemplateFile('templates/plainactionbutton.pt')

    
class ManageActionButton(ActionButton):
    """A base for 'edit' buttons
    """
    grok.baseclass()
    grok.order(2)
    grok.require('waeup.manageUniversity')
    icon = 'actionicon_modify.png'
    target = '@@manage'
    text = 'Edit'

class AddActionButton(ActionButton):
    """A base for 'add' buttons.
    """
    grok.baseclass()
    grok.order(4)
    grok.require('waeup.manageUniversity')
    icon = 'actionicon_add.png'
    target = 'add'
    text = 'Add'
    
class RemoveActionButton(ActionButton):
    """A base for 'remove' buttons.
    """
    grok.baseclass()
    grok.order(4)
    grok.require('waeup.manageUniversity')
    icon = 'actionicon_delete.png'
    target = 'remove'
    text = 'Remove'    

class SearchActionButton(ActionButton):
    """A base for 'search' buttons.
    """
    grok.baseclass()
    grok.order(5)
    grok.require('waeup.manageUniversity')
    icon = 'actionicon_search.png'
    target = 'search'
    text = 'Search'


#
# General viewlets (for more than one page/context)
#

class BreadCrumbs(grok.Viewlet):
    grok.context(IWAeUPObject)
    grok.viewletmanager(BreadCrumbManager)
    grok.order(1)

    def getEntries(self):
        result = []
        site = grok.getSite()
        context = self.context
        breadcrumbs = IBreadcrumbContainer(self.view)
        for breadcrumb in breadcrumbs:
            yield dict(
                title = breadcrumb.title,
                url = self.view.url(breadcrumb.context, breadcrumb.viewname)
                )


# Problem with circular references. Disabled for now...
# class ExportXMLAction(grok.Viewlet):
#    grok.viewletmanager(ActionBar)
#     #grok.view(Index)
#     grok.order(98)
#     grok.require('waeup.manageUniversity')

#class ImportXMLAction(grok.Viewlet):
#    grok.viewletmanager(ActionBar)
#    #grok.view(Index)
#    grok.order(99)
#    grok.require('waeup.manageUniversity')
#
#    def update(self):
#        # We cannot simply replace local sites.
#        self.can_import = not ISite.providedBy(self.context)

class LeftSidebarLink(grok.Viewlet):
    """ An entry on left sidebar.

    This is only a baseclass that won't be rendered actually. Deriving
    viewlets can override certain values and will be rendered with the
    values set here as default.
    """
    grok.baseclass()
    grok.viewletmanager(LeftSidebar)
    grok.context(IWAeUPObject)
    grok.order(5)
    grok.require('waeup.manageUniversity')
    icon = 'actionicon_modify.png' # File must exist in static/
    title = 'Text of link'

    @property
    def url(self):
        return '@@index'

    @property
    def icon_url(self):
        """Get the icon URL.
        """
        if self.icon:
            static = self.view.static
            if static is None or static.get(self.icon, None) is None:
                # In derived classes defined in other modules/packages
                # than w.s.browser, ``static`` might refer to a static dir
                # local to the derived class' module. As we often like to
                # get the icons from here
                # (i.e. waeup.sirp.browser/static), we set the directory
                # resource appropiately.
                #
                # XXX: The hardcoding of 'w.s.browser' should be replaced
                #      by something smarter.
                #
                # TODO: notes in here should go to general documentation.
                static = queryAdapter(
                    self.request, Interface, name='waeup.sirp.browser')
            return static[self.icon]()
        return
    
    # Render link only if url is provided.
    def render(self):
        if self.url:
            if self.icon_url:
                return u'<div class="portlet"><a href="%s"><img src="%s" /> %s </a></div>' % (
                    self.url, self.icon_url, self.title)
            else:
                return u'<div class="portlet"><a href="%s">%s </a></div>' % (
                    self.url, self.title)
        else:
            return ''
            
#
# waeup.sirp.app.University viewlets...
#
class Login(grok.Viewlet):
    """This viewlet allows to login in the sidebar.
    """
    grok.viewletmanager(LeftSidebar)
    grok.context(IWAeUPObject)
    grok.view(Interface)
    grok.order(2)

    def isAuthenticated(self):
        """Is the calling user authenticated?
        """
        return self.request.principal.id != 'zope.anybody'

class ManageLink(grok.Viewlet):
    """A link displayed in the upper left box.

    This viewlet renders a link to the application object's settings
    form (the 'manage' view).

    In derived classes you can create different links by setting a
    different link and text attribute. The `link` parameter is
    understood relative to the respective application object, so that
    ``@@manage`` will create a link to
    ``localhost:8080/app/@@manage``.

    Links defined by descendants from this viewlet are displayed on
    every page the user is allowed to go to, if the user has also the
    permissions set by `grok.require()`. By default only users with
    ``waeup.manageUniversity`` permission will see links defined by
    this or derivated classes.
    """
    grok.viewletmanager(LeftSidebar)
    grok.context(IWAeUPObject)
    grok.view(Interface)
    grok.order(5)
    # This link is only displayed, if the user is
    # allowed to use it!
    grok.require('waeup.managePortalSettings')

    link = '@@manage'
    text = u'Portal Settings'
    
    def render(self):
        url = self.view.url(grok.getSite(), self.link)
        return u'<div class="portlet"><a href="%s">%s</a></div>' % (
                url, self.text)

class ManageUsersLink(ManageLink):
    """A link to users management, placed in upper left box.
    """
    grok.order(6)
    grok.require('waeup.manageUsers')

    link = u'users'
    text = u'Portal Users'

class ManageDataCenter(ManageLink):
    """A link to datacenter, placed in upper left box.
    """
    grok.order(6)
    grok.require('waeup.manageDataCenter')

    link = u'datacenter'
    text = u'Data Center'

#
# Manage ("Edit settings") actions...
#
class ManageUniversityActionButton(ManageActionButton):
    """ 'Edit' button for University instances.
    """
    grok.context(IUniversity)
    grok.view(UniversityPage)
    text = 'Edit portal settings'
    
#class ManageFacultiesActionButton(ManageActionButton):
#    """ 'Edit settings' button for faculty container.
#    """
#    grok.context(IFacultyContainer)
#    grok.view(FacultyContainerPage)

class ManageDataCenterActionButton(ManageActionButton):
    """ 'Edit settings' button for datacenter.
    """
    grok.context(IDataCenter)
    grok.view(DatacenterPage)
    text = 'Edit settings'

class ManageFacultyContainerActionButton(ManageActionButton):
    """ 'Manage settings' button for faculties.
    """
    grok.context(IFacultyContainer)
    grok.view(FacultyContainerPage)
    text = 'Manage academic section'

class SearchFacultyContainerActionButton(ManageActionButton):
    """ 'Manage settings' button for faculties.
    """
    grok.context(IFacultyContainer)
    grok.view(FacultyContainerPage)
    text = 'Search academic section'
    icon = 'actionicon_search.png'
    target = '@@search'
    
class ManageFacultyActionButton(ManageActionButton):
    """ 'Manage settings' button for faculties.
    """
    grok.context(IFaculty)
    grok.view(FacultyPage)
    text = 'Manage faculty'

class ManageDepartmentActionButton(ManageActionButton):
    """ 'Manage settings' button for departments.
    """
    grok.context(IDepartment)
    grok.view(DepartmentPage)
    text = 'Manage department'

class ManageCourseActionButton(ManageActionButton):
    """ 'Edit settings' button for courses.
    """
    grok.context(ICourse)
    grok.view(CoursePage)
    text = 'Edit course'
    
class ManageCertificateActionButton(ManageActionButton):
    """ 'Manage settings' button for certificates.
    """
    grok.context(ICertificate)
    grok.view(CertificatePage)
    text = 'Manage certificate'

class ManageCertificateCourseActionButton(ManageActionButton):
    """ 'Manage settings' button for certificate courses.
    """
    grok.context(ICertificateCourse)
    grok.view(CertificateCoursePage)
    text = 'Edit course referrer'

#
# Add actions...
#

class AddUserActionButton(AddActionButton):
    grok.require('waeup.manageUsers')
    grok.context(IUserContainer)
    grok.view(UserContainerPage)
    text = 'Add user'

#class AddFacultyActionButton(AddActionButton):
#    grok.context(IFacultyContainer)
#    grok.view(FacultyContainerPage)
#    text = 'Add faculty'
    
#class AddDepartmentActionButton(AddActionButton):
#    grok.context(IFaculty)
#    grok.view(FacultyPage)
#    text = 'Add department'

#class AddCertificateActionButton(AddActionButton):
#    grok.context(IDepartment)
#    grok.view(DepartmentPage)
#    grok.order(3)
#    text = 'Add certificate'
#    target = 'addcertificate'
    
#class AddCourseActionButton(AddActionButton):
#    grok.context(IDepartment)
#    grok.view(DepartmentPage)
#    grok.order(4)
#    text = 'Add course'
#    target = 'addcourse'

#class AddCertificateCourseActionButton(AddActionButton):
#    grok.context(ICertificate)
#    grok.view(CertificatePage)
#    grok.order(4)
#    text = 'Add course referrer'
#    target = 'addcertificatecourse'

#
# Remove actions...
#


#class RemoveFacultyActionButton(RemoveActionButton):
#    grok.context(IFacultyContainer)
#    grok.view(FacultyContainerPage)
#    text = 'Remove faculty'
   
#class RemoveDepartmentActionButton(RemoveActionButton):
#    grok.context(IFaculty)
#    grok.view(FacultyPage)
#    text = 'Remove department'
    

#
# Actions with a 'browse' icon...
#
class BrowseActionButton(ActionButton):
    grok.baseclass()
    grok.context(IWAeUPObject)
    grok.template('actionbutton')
    grok.viewletmanager(ActionBar)
    grok.require('waeup.manageUniversity')
    icon = 'actionicon_manage.png' # File must exist in static/
    target = '@@show' # link to this viewname.
    text = 'Show batch logs' # Text to display on the button

class BrowseDatacenterLogs(BrowseActionButton):
    grok.context(IDataCenter)
    grok.view(DatacenterPage)
    grok.order(4)
    icon = 'documentinfo_templet.png'
    target = '@@logs'
    text = 'Show batch logs'

#
# Misc. buttons...
#
class BatchOpButton(ActionButton):
    grok.context(IDataCenter)
    grok.view(DatacenterPage)
    grok.require('waeup.manageUniversity')
    grok.order(6)
    icon = 'actionbox_templet.png'
    target = '@@import1'
    text = 'Batch processing'

class UploadCSVButton(ActionButton):
    grok.context(IDataCenter)
    grok.view(DatacenterPage)
    grok.require('waeup.manageUniversity')
    grok.order(5)
    icon = 'go-up-16x16.png'
    target = '@@upload'
    text = 'Upload CSV file'

#
# Primary navigation tabs (in upper left navigation bar)...
#
class PrimaryNavManager(grok.ViewletManager):
    """Viewlet manager for the primary navigation tab.
    """
    grok.name('primary_nav')

class PrimaryNavTab(grok.Viewlet):
    """Base for primary nav tabs.
    """
    grok.baseclass()
    grok.viewletmanager(PrimaryNavManager)
    grok.order(1)
    grok.require('waeup.View')

    pnav = 0 # This is a kind of id of a tab. If some page provides
             # also a 'pnav' attribute with the same value (here: 0),
             # then the tab will be rendered as 'active' when the page
             # gets rendered.
             #
             # This way you can assign certain pages to certain
             # primary nav tabs. Each primary tab should therefore set
             # the 'pnav' attribute to a different value (or several
             # tabs might be rendered as active simultanously when the
             # page gets rendered.
    tab_title = u'Some Text'
    
    @property
    def link_target(self):
        return self.view.application_url()

    @property
    def active(self):
        view_pnav = getattr(self.view, 'pnav', 0)
        if view_pnav == self.pnav:
            return 'active'
        return ''

class HomeTab(PrimaryNavTab):
    """Home-tab in primary navigation.
    """
    grok.order(1)
    grok.require('waeup.Public')
    grok.template('primarynavtab')

    pnav = 0
    tab_title = u'Home'

class FacultiesTab(PrimaryNavTab):
    """Faculties-tab in primary navigation.
    """
    grok.order(2)
    grok.require('waeup.View')
    grok.template('primarynavtab')

    pnav = 1
    tab_title = u'Academics'

    @property
    def link_target(self):
        return self.view.application_url('faculties')

    
class ContactTab(PrimaryNavTab):
    """Contact tab in primary navigation.
    """
    grok.order(4)
    grok.require('waeup.Anonymous')
    grok.template('primarynavtab')
    
    pnav = 2
    def tab_title(self):
        """Display tab only for anonymous. Authenticated users can call the 
        form from the user navigation bar.
        """
        userid = self.request.principal.id
        if userid != 'zope.anybody':
            tt = u'' 
        else:
            tt = u'Enquiries'         
        return tt 

    @property
    def active(self):
        view_pnav = getattr(self.view, 'pnav', 0)
        userid = self.request.principal.id
        if view_pnav == self.pnav and userid == 'zope.anybody':
            return 'active'
        return ''

    @property
    def link_target(self):
        return self.view.application_url('contactadmin')        



#
# Administration tasks
#
class AdminTask(grok.Viewlet):
    """The base for task entries on administration page.
    """
    grok.baseclass()
    grok.order(1)
    grok.viewletmanager(AdministrationTasks)
    grok.require('waeup.manageUniversity')
    grok.template('admintask')

    link_title = 'Manage users' # How the link to the target will be titled.
    target_viewname = 'users'   # The name of the target view.
    
    @property
    def link_target(self):
        return self.view.url(self.context[self.target_viewname])

class AdminTaskPortalSettings(AdminTask):
    """Entry on administration page that link to portal settings.
    """
    grok.order(1)
    grok.require('waeup.managePortalSettings')

    link_title = 'Portal Settings'
    def link_target(self):
        return self.view.url(self.view.context, '@@manage')

class AdminTaskUsers(AdminTask):
    """Entry on administration page that link to user folder.
    """
    grok.order(2)
    grok.require('waeup.manageUsers')

    link_title = 'Portal Users'
    target_viewname = 'users'
    
class AdminTaskDatacenter(AdminTask):
    """Entry on administration page that link to datacenter.
    """
    grok.order(3)
    grok.require('waeup.manageDataCenter')

    link_title = 'Data Center'
    target_viewname = 'datacenter'

class SubobjectLister(grok.ViewletManager):
    """Very special viewlet manager that displays lists of subobjects.
    """
    grok.name('subobjectlist')
    grok.template('subobjectlist')

    def update(self):
        # The default implementation of update() sets self.viewlets to
        # a list of viewlets for the current context
        # (self.context). We make use of that fact by retrieving all
        # viewlets for all items in our context container by simply
        # setting these items as context while we call the default
        # update() method. So we get a list of lists of viewlets for
        # each item in a 'row' (where a single item is a row).
        rows = []
        orig_context = self.context
        for name, value in self.context.items():
            # Retrieve all viewlets for the current item (not the context)
            self.context = value
            super(SubobjectLister, self).update() # sets self.viewlets
            rows.append(self.viewlets)
            self.context = orig_context
        self.rows = rows
        # Finally, set the viewlets we would retrieve normally...
        super(SubobjectLister, self).update()
        return


class FacultyContainerListHead(grok.Viewlet):
    """The header line of faculty container subobject lists.
    """
    grok.order(1)
    grok.viewletmanager(SubobjectLister)
    grok.context(IFacultyContainer)
    grok.require('waeup.View')

    def render(self):
        return u'<th>Code</th><th>Title</th><th></th>'

class FacultyListName(grok.Viewlet):
    """Display a the title of a faculty as link in a list.
    """
    grok.order(1)
    grok.viewletmanager(SubobjectLister)
    grok.context(IFaculty)
    grok.require('waeup.View')

    def render(self):
        return u'<a href="%s">%s</a>' % (
            self.view.url(self.context), self.context.__name__)

class FacultyListTitle(grok.Viewlet):
    """Display the title of a faculty in a list.
    """
    grok.order(2)
    grok.viewletmanager(SubobjectLister)
    grok.context(IFaculty)
    grok.require('waeup.View')

    def render(self):
        return self.context.title

class FacultyRemoveButton(grok.Viewlet):
    """Render a remove button for faculty lists.
    """
    grok.order(3)
    grok.viewletmanager(SubobjectLister)
    grok.context(IFaculty)
    grok.require('waeup.manageUniversity')

    def render(self):
        return u'<div class="text-right"><form method="POST"><input class="text-right" type="submit" value="remove" /></form></div>'
